@@ -7,9 +7,8 @@ from pyravendb.tools.utils import Utils
 from abc import abstractmethod
 from datetime import timedelta
 import collections
-import string
-import random
 import logging
+import uuid
 
 logging.basicConfig(filename='responses.log', level=logging.DEBUG)
 log = logging.getLogger()
@@ -18,7 +17,7 @@ log = logging.getLogger()
 # Todo update the commands
 class RavenCommand(object):
     def __init__(self, url=None, method=None, data=None, files=None, headers=None, is_read_request=False,
-                 use_stream=False):
+                 is_raft_request=False, use_stream=False):
         self.url = url
         self.method = method
         self.data = data
@@ -32,6 +31,10 @@ class RavenCommand(object):
         self.timeout = None
         self.requested_node = None
         self.files = files
+        self.is_raft_request = is_raft_request
+        self._raft_unique_request_id = None
+        if is_raft_request:
+            self._raft_unique_request_id = uuid.uuid4()
 
     @abstractmethod
     def create_request(self, server_node):
@@ -45,6 +48,10 @@ class RavenCommand(object):
     def raven_command(self):
         return self.__raven_command
 
+    @property
+    def raft_unique_request_id(self):
+        return str(self._raft_unique_request_id)
+
     def is_failed_with_node(self, node):
         return len(self.failed_nodes) > 0 and node in self.failed_nodes
 
@@ -202,7 +209,7 @@ class BatchCommand(RavenCommand):
         try:
             response = response.json()
             if "Error" in response:
-                raise ValueError(response["Error"])
+                raise ValueError(response["Message"])
             return response["Results"]
         except ValueError as e:
             raise exceptions.InvalidOperationException(e)
@@ -214,7 +221,7 @@ class DeleteIndexCommand(RavenCommand):
         @param str index_name: Name of the index you like to get or delete
         @param str database_name: Name of the index database
         """
-        super(DeleteIndexCommand, self).__init__(method="DELETE")
+        super(DeleteIndexCommand, self).__init__(method="DELETE", is_raft_request=True)
         self.index_name = index_name
         self.database_name = database_name
 
@@ -505,6 +512,9 @@ class GetDatabaseRecordCommand(RavenCommand):
         self.url = "{0}/admin/databases?name={1}".format(server_node.url, self._database_name)
 
     def set_response(self, response):
+        if response is None:
+            return None
+
         try:
             response = response.json()
             if "Error" in response:
@@ -586,7 +596,7 @@ class CreateSubscriptionCommand(RavenCommand):
         @param SubscriptionCreationOptions options: Subscription options
         """
 
-        super(CreateSubscriptionCommand, self).__init__(method="PUT")
+        super(CreateSubscriptionCommand, self).__init__(method="PUT", is_raft_request=True)
         self._options = options
 
     def create_request(self, server_node):
@@ -605,7 +615,7 @@ class CreateSubscriptionCommand(RavenCommand):
 
 class DeleteSubscriptionCommand(RavenCommand):
     def __init__(self, name):
-        super(DeleteSubscriptionCommand, self).__init__(method="DELETE")
+        super(DeleteSubscriptionCommand, self).__init__(method="DELETE", is_raft_request=True)
         self._name = name
 
     def create_request(self, server_node):
@@ -618,7 +628,7 @@ class DeleteSubscriptionCommand(RavenCommand):
 
 class DropSubscriptionConnectionCommand(RavenCommand):
     def __init__(self, name):
-        super(DropSubscriptionConnectionCommand, self).__init__(method="POST")
+        super(DropSubscriptionConnectionCommand, self).__init__(method="POST", is_raft_request=True)
         self._name = name
 
     def create_request(self, server_node):
