@@ -1,3 +1,5 @@
+from requests_pkcs12 import Pkcs12Adapter
+
 from pyravendb.commands.raven_commands import GetTopologyCommand, GetStatisticsCommand
 from pyravendb.connection.requests_helpers import *
 from pyravendb.custom_exceptions import exceptions
@@ -31,9 +33,18 @@ class RequestsExecutor(object):
                 raise
         return instance
 
+    @staticmethod
+    def initialize_certificate(certificate):
+        if not isinstance(certificate, dict):
+            return certificate, None
+        pfx = certificate["pfx"]
+        password = certificate.get("password", None)
+        adapter = Pkcs12Adapter(pkcs12_data=pfx, pkcs12_password=password)
+        return None, adapter
+
     def __init__(self, database_name, certificate, conventions=None, **kwargs):
         self._database_name = database_name
-        self._certificate = certificate
+        (self._certificate, self._adapter) = self.initialize_certificate(certificate)
         self.topology_etag = kwargs.get("topology_etag", 0)
         self._last_return_response = datetime.utcnow()
         self.conventions = conventions if conventions is not None else DocumentConventions()
@@ -41,7 +52,7 @@ class RequestsExecutor(object):
         self._last_known_urls = None
 
         self.headers = {"Accept": "application/json",
-                        "Raven-Client-Version": "4.0.0.0"}
+                        "Raven-Client-Version": "5.0.0.1"}
 
         self.update_topology_lock = Lock()
         self.update_timer_lock = Lock()
@@ -123,6 +134,11 @@ class RequestsExecutor(object):
             node_index = 0 if self._node_selector is None else self._node_selector.current_node_index
 
             with requests.session() as session:
+                if self._adapter is not None:
+                    session.mount("https://", self._adapter)
+                if raven_command.is_raft_request:
+                    prefix = '&' if '?' in raven_command.url else '?'
+                    raven_command.url += f"{prefix}raft-request-id={raven_command.raft_unique_request_id}"
                 raven_command.headers.update(self.headers)
                 if not self._disable_topology_updates:
                     raven_command.headers["Topology-Etag"] = "\"{0}\"".format(self.topology_etag)
@@ -166,7 +182,11 @@ class RequestsExecutor(object):
                     return raven_command.set_response(None)
                 if response.status_code == 403:
                     if self._certificate is not None:
-                        with open(self._certificate, 'rb') as pem:
+                        cert = self._certificate
+                        if isinstance(cert, tuple):
+                            (cert, _) = cert
+
+                        with open(cert, 'rb') as pem:
                             cert = crypto.load_certificate(crypto.FILETYPE_PEM, pem.read())
                             name = str(cert.get_subject().get_components()[0][1])
                     raise exceptions.AuthorizationException(
