@@ -51,7 +51,7 @@ class CreateDatabaseOperation(ServerOperation):
         def __init__(self, database_record, replication_factor, convention):
             if convention is None:
                 raise ValueError("Invalid convention")
-            super(CreateDatabaseOperation._CreateDatabaseCommand, self).__init__(method="PUT")
+            super(CreateDatabaseOperation._CreateDatabaseCommand, self).__init__(method="PUT", is_raft_request=True)
 
             self._database_record = database_record
             self._replication_factor = replication_factor
@@ -59,21 +59,20 @@ class CreateDatabaseOperation(ServerOperation):
 
         def create_request(self, server_node):
             self.url = "{0}/admin/databases?name={1}".format(server_node.url, self._database_record["DatabaseName"])
-            self.url += "&replication-factor=" + str(self._replication_factor)
+            self.url += "&replicationFactor=" + str(self._replication_factor)
 
             self.data = self._database_record
 
         def set_response(self, response):
             if response is None:
-                raise ValueError("response is invalid.")
+                raise ValueError("Invalid response")
 
-            if response.status_code == 201:
-                return response.json()
-
-            if response.status_code == 400:
+            try:
                 response = response.json()
                 if "Error" in response:
-                    raise RequestException(response["Error"])
+                    raise RequestException(response["Message"], response["Type"])
+            except ValueError:
+                raise response.raise_for_status()
 
 
 class DeleteDatabaseOperation(ServerOperation):
@@ -97,7 +96,7 @@ class DeleteDatabaseOperation(ServerOperation):
 
             self._parameters = parameters
 
-            super(DeleteDatabaseOperation._DeleteDatabaseCommand, self).__init__(method="DELETE")
+            super(DeleteDatabaseOperation._DeleteDatabaseCommand, self).__init__(method="DELETE", is_raft_request=True)
 
         def create_request(self, server_node):
             self.url = "{0}/admin/databases".format(server_node.url)
@@ -107,7 +106,7 @@ class DeleteDatabaseOperation(ServerOperation):
             try:
                 response = response.json()
                 if "Error" in response:
-                    raise exceptions.DatabaseDoesNotExistException(response["Message"])
+                    raise exceptions.DatabaseDoesNotExistException(response["Message"], response["Type"])
             except ValueError:
                 raise response.raise_for_status()
             return {"raft_command_index": response["RaftCommandIndex"]}
