@@ -6,6 +6,10 @@ from pyravendb.raven_operations.operations import GetAttachmentOperation
 from pyravendb.commands.commands_data import PutAttachmentCommandData, DeleteAttachmentCommandData, DeleteCommandData
 from pyravendb.tools.utils import Utils
 from pyravendb.data.operation import AttachmentType
+from pyravendb.data.timeseries import TimeSeriesRangeResult
+from .session_timeseries import TimeSeries
+from .session_counters import DocumentCounters
+from typing import Dict, List
 
 
 class _SaveChangesData(object):
@@ -31,12 +35,18 @@ class DocumentSession(object):
         self._included_documents_by_id = {}
         self._deleted_entities = set()
         self._documents_by_entity = {}
+        self._timeseries_defer_commands = {}
+        self._time_series_by_document_id = {}
+        self._counters_defer_commands = {}
+        self._counters_by_document_id = {}
         self._known_missing_ids = set()
         self.id_value = None
         self._defer_commands = set()
         self._number_of_requests_in_session = 0
         self._query = None
         self.use_optimistic_concurrency = kwargs.get("use_optimistic_concurrency", False)
+        self.no_tracking = kwargs.get("no_tracking", False)
+        self.no_caching = kwargs.get("no_caching", False)
         self.advanced = Advanced(self)
 
     def __enter__(self):
@@ -65,6 +75,14 @@ class DocumentSession(object):
     def documents_by_id(self):
         return self._documents_by_id
 
+    @property
+    def time_series_by_document_id(self) -> Dict[str, Dict[str, List[TimeSeriesRangeResult]]]:
+        return self._time_series_by_document_id
+
+    @property
+    def counters_by_document_id(self):
+        return self._counters_by_document_id
+
     def defer(self, command, *args):
         self._defer_commands.add(command)
         for arg in args:
@@ -74,6 +92,14 @@ class DocumentSession(object):
     def defer_commands(self):
         return self._defer_commands
 
+    @property
+    def counters_defer_commands(self):
+        return self._counters_defer_commands
+
+    @property
+    def timeseries_defer_commands(self):
+        return self._timeseries_defer_commands
+
     @property
     def known_missing_ids(self):
         return self._known_missing_ids
@@ -98,7 +124,7 @@ class DocumentSession(object):
                 if key not in self._documents_by_id:
                     self._included_documents_by_id[key] = value
 
-    def save_entity(self, key, entity, original_metadata, metadata, document, force_concurrency_check=False):
+    def save_entity(self, key, entity, original_metadata, metadata, original_document, force_concurrency_check=False):
         if key is not None:
             self._known_missing_ids.discard(key)
 
@@ -106,15 +132,17 @@ class DocumentSession(object):
                 self._documents_by_id[key] = entity
 
                 self._documents_by_entity[self._documents_by_id[key]] = {
-                    "original_value": document.copy(), "metadata": metadata,
+                    "original_value": original_document, "metadata": metadata,
                     "original_metadata": original_metadata, "change_vector": metadata.get("change_vector", None),
                     "key": key, "force_concurrency_check": force_concurrency_check}
 
     def _convert_and_save_entity(self, key, document, object_type, nested_object_types):
+
         if key not in self._documents_by_id:
-            entity, metadata, original_metadata = Utils.convert_to_entity(document, object_type, self.conventions,
-                                                                          nested_object_types)
-            self.save_entity(key, entity, original_metadata, metadata, document)
+            entity, metadata, original_metadata, original_document = Utils.convert_to_entity(document, object_type,
+                                                                                             self.conventions,
+                                                                                             nested_object_types)
+            self.save_entity(key, entity, original_metadata, metadata, original_document)
 
     def _multi_load(self, keys, object_type, includes, nested_object_types):
         if len(keys) == 0:
@@ -251,7 +279,7 @@ class DocumentSession(object):
         :type object:
         @param key: Entity will be stored under this key, (None to generate automatically)
         :type str:
-        @param change_vector: Current entity change_vector, used for concurrency checks (null to skip check)
+        @param change_vector: Current entity change_vector, used for concurrency checks (None to skip check)
         :type str
         """
 
@@ -302,6 +330,18 @@ class DocumentSession(object):
 
         return "disabled" if change_vector is None else "forced"
 
+    def time_series_for(self, entity_or_document_id, name):
+        """
+        Get A time series object associated with the document
+        """
+        return TimeSeries(self, entity_or_document_id, name)
+
+    def counters_for(self, entity_or_document_id):
+        """
+        Get A counters object associated with the document
+        """
+        return DocumentCounters(self, entity_or_document_id)
+
     def save_changes(self):
         data = _SaveChangesData(list(self._defer_commands), len(self._defer_commands))
         self._defer_commands.clear()
@@ -319,20 +359,34 @@ class DocumentSession(object):
 
     def _update_batch_result(self, batch_result, data):
 
-        i = data.deferred_command_count
+        i = 0
         batch_result_length = len(batch_result)
         while i < batch_result_length:
             item = batch_result[i]
-            if item["Type"] == "PUT":
-                entity = data.entities[i - data.deferred_command_count]
-                if entity in self._documents_by_entity:
-                    self._documents_by_id[item["@id"]] = entity
-                    document_info = self._documents_by_entity[entity]
-                    document_info["change_vector"] = item["@change-vector"]
-                    item.pop("Type", None)
-                    document_info["original_metadata"] = item.copy()
-                    document_info["metadata"] = item
-                    document_info["original_value"] = entity.__dict__.copy()
+            if isinstance(item, dict) and "Type" in item:
+                if item["Type"] == "PUT":
+                    entity = data.entities[i - data.deferred_command_count]
+                    if entity in self._documents_by_entity:
+                        self._documents_by_id[item["@id"]] = entity
+                        document_info = self._documents_by_entity[entity]
+                        document_info["change_vector"] = item["@change-vector"]
+                        item.pop("Type", None)
+                        document_info["original_metadata"] = item.copy()
+                        document_info["metadata"] = item
+                        document_info["key"] = item["@id"]
+                        document_info["original_value"] = entity.__dict__.copy()
+                elif item["Type"] == "Counters":
+                    cache = self._counters_by_document_id.get(item["Id"], None)
+                    if not cache:
+                        self._counters_by_document_id[item["Id"]] = [False, {}]
+                        cache = self._counters_by_document_id[item["Id"]]
+
+                    for counter in item["CountersDetail"]["Counters"]:
+                        cache[1][counter["CounterName"]] = counter["TotalValue"]
+
+                    self._counters_defer_commands.clear()
+                elif item["Type"] == "TimeSeries":
+                    self._timeseries_defer_commands.clear()
             i += 1
 
     def _prepare_for_delete_commands(self, data):
@@ -371,8 +425,10 @@ class DocumentSession(object):
                 data.commands.append(commands_data.PutCommandData(key, change_vector, document, metadata))
 
     def _has_change(self, entity):
-        if self._documents_by_entity[entity]["original_metadata"] != self._documents_by_entity[entity]["metadata"] \
-                or self._documents_by_entity[entity]["original_value"] != entity.__dict__:
+        import json
+        entity_to_dict = json.loads(json.dumps(entity, default=self.conventions.json_default_method))
+        if self._documents_by_entity[entity]["original_value"] != entity_to_dict or self._documents_by_entity[entity][
+            "original_metadata"] != self._documents_by_entity[entity]["metadata"]:
             return True
         return False
 
@@ -420,8 +476,8 @@ class Advanced(object):
 
             results = ijson.backend.common.items(parser, "Results")
             for result in next(results, None):
-                document, metadata, _ = Utils.convert_to_entity(result, query.object_type, self.session.conventions,
-                                                                query.nested_object_types)
+                document, metadata, _, _ = Utils.convert_to_entity(result, query.object_type, self.session.conventions,
+                                                                   query.nested_object_types)
                 yield {"document": document, "metadata": metadata, "id": metadata.get("@id", None),
                        "change-vector": metadata.get("@change-vector", None)}
 
@@ -466,7 +522,7 @@ class _Attachment:
     def throw_not_in_session(self, entity):
         raise ValueError(
             repr(entity) + " is not associated with the session, cannot add attachment to it. "
-                           "Use documentId instead or track the entity in the session.")
+                           "Use document Id instead or track the entity in the session.")
 
     def store(self, entity_or_document_id, name, stream, content_type=None, change_vector=None):
         if not isinstance(entity_or_document_id, str):
