@@ -41,7 +41,6 @@ class Query(object):
         self._current_clause_depth = None
         self.is_intersect = None
         self.the_wait_for_non_stale_results = False
-        self.fetch = None
         self._query = None
         self.start = None
         self.page_size = None
@@ -92,7 +91,6 @@ class Query(object):
         self.cutoff_etag = None
         self.wait_for_non_stale_results = wait_for_non_stale_results
         self.timeout = timeout if timeout is not None else self.session.conventions.timeout
-        self.fetch = None
         self._query = None
         self.last_equality = None
         self.is_distinct = False
@@ -276,7 +274,7 @@ class Query(object):
             write = "".join(write_builder)
 
         if write is None:
-            raise AttributeError("token.token don't match any of the cases for rql builder")
+            raise AttributeError(f"{token.token} don't match any of the cases for rql builder")
         return write
 
     @staticmethod
@@ -434,11 +432,15 @@ class Query(object):
         return self.where_equals(field_name, value, exact)
 
     def where_exists(self, field_name):
+        if field_name is None:
+            raise ValueError("None field_name is invalid")
+
         field_name = Query.escape_if_needed(field_name)
 
         self._add_operator_if_needed()
         self.negate_if_needed(field_name)
-        self._where_tokens.append(_Token(field_name=field_name, value=None, token="exists", write=")"))
+        self._where_tokens.append(_Token(field_name=field_name, value=None, token="exists", write="exists(" +
+                                                                                                  field_name + ")"))
         return self
 
     def where_true(self):
@@ -799,13 +801,13 @@ class Query(object):
         response_includes = response.pop("Includes", None)
         self.session.save_includes(response_includes)
         for result in response_results:
-            entity, metadata, original_metadata = Utils.convert_to_entity(result, self.object_type, conventions,
-                                                                          self.nested_object_types,
-                                                                          fetch=False if not self.fetch else True)
+            entity, metadata, original_metadata, original_document = Utils.convert_to_entity(result, self.object_type,
+                                                                                             conventions,
+                                                                                             self.nested_object_types)
             if self.object_type != dict and not self.fields_to_fetch:
                 self.session.save_entity(key=original_metadata.get("@id", None), entity=entity,
                                          original_metadata=original_metadata,
-                                         metadata=metadata, document=result)
+                                         metadata=metadata, original_document=original_document)
             results.append(entity)
 
         if self._with_statistics:
